/*
 *  TOPPERS/DeviceManager
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      DeviceManager
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2009 by Embedded and Real-Time Systems Laboratory
 *            Graduate School of Information Science, Nagoya Univ., JAPAN  
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 *
 *  @(#) $Id: kernel.cpp,v 1.7 2003/12/24 07:40:42 takayuki Exp $
 */

#include "stdafx.h"
#include "devicemanager.h"
#include "kernel.h"

/////////////////////////////////////////////////////////////////////////////
// CKernel

CKernel::CKernel()
{
	KerID = _Module.AddKernel(this);
}

CKernel::~CKernel()
{
	_Module.DeleteKernel(this);
}

STDMETHODIMP CKernel::IsValid(BOOL *valid)
{
	*valid = (KerID != 0) ? TRUE : FALSE;
	return S_OK;
}

STDMETHODIMP CKernel::Read(unsigned long address, unsigned long sz, unsigned char data[])
{
	if(KerID!=0 && _Module.Read(this,address,sz,data))
		return S_OK;
	return E_FAIL;
}

STDMETHODIMP CKernel::Write(unsigned long address, unsigned long sz, unsigned char data[])
{
	if(KerID!=0 && _Module.Write(this,address,sz,data))
		return S_OK;
	return E_FAIL;
}

STDMETHODIMP CKernel::OnLogEvent(long sz, unsigned char data[])
{
	if(KerID!=0 && _Module.OnLogEvent(this,sz,data))
		return S_OK;
	return E_FAIL;
}

STDMETHODIMP CKernel::Map(unsigned long id)
{
	if(KerID!=0 && _Module.MapKernel(this,id))
		return S_OK;
	return E_FAIL;
}

STDMETHODIMP CKernel::Unmap(unsigned long id)
{
	if(KerID!=0 && _Module.UnmapKernel(this,id))
		return S_OK;
	return E_FAIL;
}

STDMETHODIMP CKernel::RaiseInterrupt(unsigned long inhno)
{
	if(KerID == 0)
		return E_HANDLE;
	if(_Module.KernelList.empty()) //K؂ȃJ[l𒲂ׂׂH
		return E_ACCESSDENIED;
	if(_Module.RaiseInterrupt(inhno) == false)
		return E_INVALIDARG;
	return S_OK;
}

STDMETHODIMP CKernel::CreateSharedMemory(unsigned long address, unsigned long size, BSTR lpName)
{
	if(KerID == 0)
		return E_HANDLE;
	if(_Module.CreateSharedMemory(this, address, size, lpName) == false)
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CKernel::ReleaseSharedMemory(BSTR lpName)
{
	if(KerID == 0)
		return E_HANDLE;
	if(_Module.ReleaseSharedMemory(this, lpName) == false)
		return E_INVALIDARG;
	return S_OK;
}

STDMETHODIMP CKernel::CreateSharedMemoryAnyName(unsigned long address, unsigned long size, BSTR * name)
{
	if(KerID == 0)
		return E_HANDLE;
	if(( *name = ::SysAllocString(_Module.CreateSharedMemoryAnyName(this, address, size)) ) == NULL)
		return E_FAIL;
	return S_OK;
}

/*
STDMETHODIMP CKernel::ReleaseSharedMemory2(unsigned long address)
{
	if(KerID == 0)
		return E_HANDLE;
	if(_Module.ReleaseSharedMemory2(address) == false)
		return E_INVALIDARG;
	return S_OK;
}
*/

STDMETHODIMP CKernel::AcquireLock(unsigned long address, unsigned long size)
{
	if(KerID == 0)
		return E_HANDLE;
	result = false;
	//MessageBox(NULL, _T("Acquire Lock"), _T("debug"), MB_OK);
	result = _Module.AcquireLock(address, size);
	if(result == false)
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CKernel::ReleaseLock(unsigned long address, unsigned long size)
{
	if(KerID == 0)
		return E_HANDLE;
	result = false;
	result = _Module.ReleaseLock(address, size);
	if(result == false)
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CKernel::CreateMutex(unsigned long number, BSTR * name)
{
	if( ( *name = ::SysAllocString(_Module.CreateMutex(number, this)) ) == NULL )
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CKernel::CloseMutex(unsigned long number)
{
	_Module.CloseMutex(number, this);
	return S_OK;
}

STDMETHODIMP CKernel::GetMemory(BSTR lpName, unsigned long * address, unsigned long * size)
{
	bool res;
	res = _Module.GetMemory(lpName, address, size);
	if(res == false)
		return E_FAIL;
	return S_OK;
}
